/*
 Copyright (C) 2007 by Ben Anderman <ben@happyspork.com>
 Part of Sporktris http://www.happyspork.com/games
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License.
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY.
 
 See the License.txt file for more details.
 */

#include "sporktris.h"
#include "keys.h"
#include "layers.h"
#include "pieces.h"
#include "namedv.h"
#include "prefs_file.h"
#include "common_funcs.h"
#include <SDL.h>
#include <SDL_image.h>
#include <vector>
#include <string>
#include <stdio.h>
#include <iostream>
#include <algorithm>
#include <math.h>
#include <ctime>
#include <fstream>
#include <cstdlib>

using namespace std;

bool Sporktris::game_loop (void)
{
	random(0,1);
	set_level(user_prefs->geti("start_level"));
	SDL_Event event;
	cur_piece = rand_piece();
	next_piece = rand_piece();
	update_next_piece();
	update_level();
	update_points();
	update_lines();
	init_keys();
	make_garbage(user_prefs->geti("garbage_lines"));
	bool playing = true;
	bool exit = false;
	update_win();
	last_step = SDL_GetTicks();
	while (playing)
	{
		bool redraw = false;
		if (SDL_PollEvent(&event) && playing == true)
		{
			int result = handle_events(&event);
			if (result) {playing = false; exit = true;}
		}
		
		int input_result = handle_input();
		if (input_result == 1) redraw = true;
		if (input_result == 2) {playing = false; redraw = true;}
		if (paused == false && !(SDL_GetAppState() & SDL_APPINPUTFOCUS))
		{
			pause();
		}
		
		while (paused == false && last_step+speed < SDL_GetTicks())
		{
			if (!awaiting_gravity)
			{
				if (cur_piece.canMove(0,1))
				{
					move_piece(0, 1);
					redraw = true;
				} else {
					if (cur_piece.getTop() >= 0)
					{
						drop_piece();
						redraw = true;
					} else {
						playing = false;
					}
				}
			}else{
				gravity();
				redraw = true;
				awaiting_gravity = false;
			}
			last_step += speed;
		}
		if (redraw) update_win();
		SDL_Delay(1+100*paused);
	}
	if (!exit) exit = end_game();
	return exit;
}

bool Sporktris::end_game (void)
{
	unsigned int ended = SDL_GetTicks();
	SDL_Rect src, dest;
	SDL_GetClipRect(game_over, &src);
	dest = src;
	dest.x = (prefs->geti("board_x")+(prefs->geti("board_width")*prefs->geti("block_size"))/2)-src.w/2;
	dest.y = (prefs->geti("board_y")+(prefs->geti("board_height")*prefs->geti("block_size"))/2)-src.h/2;
	SDL_BlitSurface(game_over, &src, screen, &dest);
	SDL_Flip(screen);
	SDL_Event event;
	while (SDL_PollEvent(&event)) {} //clear events
	while (SDL_WaitEvent(&event) != 0)
	{
		if (event.type == SDL_QUIT)
		{
			return true;
		} else if (event.type == SDL_KEYDOWN)
		{
			SDLKey sym = event.key.keysym.sym;
			if (sym == SDLK_RETURN || sym == SDLK_ESCAPE || (sym == SDLK_SPACE && SDL_GetTicks() > ended+500))
			{
				return false;
			}
		} else if (event.type == SDL_MOUSEBUTTONDOWN) {
			return false;
		}
	}
	return false;
}

void Sporktris::set_board (int x, int y, block_type val)
{
	if (x >= 0 && y >= 0 && (unsigned)x < board[0].size() && (unsigned)y < board.size())
	{
		//printf("x: %i | y: %i\n", x, y);
		board[y][x] = val;
	}
}

void Sporktris::inc_speed (void)
{
	speed = (int)((speed+35)/1.5);
	//speed = (int)speed*0.712;
}

void Sporktris::drop_piece (void)
{
	while (cur_piece.canMove(0,1))
	{
		move_piece(0, 1);
	}
	for (int i = 0; i < 4; i++)
	{
		set_board(cur_piece.map[i][0]+cur_piece.x, cur_piece.map[i][1]+cur_piece.y, block_type(cur_piece.graphic, theme_id));
	}
	int oldy = cur_piece.getTop();
	cur_piece = next_piece;
	next_piece = rand_piece();
	int cleared = 0;
	for (int i = 0; i < 4; i++)
	{
		if (check_line(1, oldy+i))
		{
			for (unsigned int x = 0; x < board[oldy+i].size(); x++)
			{
				set_board(x, oldy+i, block_type(-1, theme_id));
			}
			cleared++;
			cleared_lines++;
		}
		if (cleared_lines >= level*10)
		{
			set_level(level+1);
			update_level();
		}
	}
	points += cleared*cleared*level;
	if (cleared && !check_line(board.size()-1, 0)) awaiting_gravity = true;
	update_next_piece();
	update_points();
	update_lines();
	for (map<string, Key>::iterator i = keys.keys.begin(); i != keys.keys.end(); i++)
	{
		if (i->first != "drop") i->second.reset_repeat();
	}
}

void Sporktris::set_level (int new_level)
{
	if (new_level == level+1)
	{
		inc_speed();
		level++;
	} else {
		level = 1;
		speed = base_speed;
		while (level < new_level)
		{
			inc_speed();
			level++;
		}
	}
	level_theme = themes[theme_id = (level-1)%themes.size()];
	ls.findByName("background")->surf = level_theme;
	ls.findByName("background")->changed = true;
}

bool Sporktris::gravity (void)
{
	int move_down = 0;
	int dropped = 0;
	for (int y = board.size()-1; y >= 0; y--)
	{
		if (check_line(0, y))
		{
			move_down++;
		} else if (move_down > 0) {
			for (unsigned int x = 0; x < board[y].size(); x++)
			{
				if (!cur_piece.blockOn(x, y))
				{
					board[y+move_down][x] = board[y][x];
					board[y][x].type = -1;
					dropped++;
					//printf ("%i:%i\n",x,y);
				}
			}
		}
	}
	if (dropped) {awaiting_gravity = false; return true;} else {return false;}
}

bool Sporktris::check_line (bool type, int line)
{
	if (line < 0 || (unsigned)line >= board.size()) return false;
	for (unsigned int i = 0; i < board[line].size(); i++)
	{
		if (cur_piece.blockOn(i, line))
		{
			return false;
		}
		if ((board[line][i].type == -1 && type == 1) || (board[line][i].type != -1 && type == 0))
		{
			return (false);
		}
	}
	return (true);
}

void Sporktris::update_next_piece (void)
{
	SDL_Rect rect;
	SDL_GetClipRect(next_piece_box, &rect);
	clear_surf(next_piece_box);
	SDL_Rect src, dest;
	int block_size = prefs->geti("block_size");
	int lx, hx, ly, hy, cx, cy;
	lx = 3;
	ly = 3;
	hx = 0;
	hy = 0;
	for (int i = 0; i < 4; i++)
	{
		if (next_piece.map[i][0] < lx) lx = next_piece.map[i][0];
		if (next_piece.map[i][0] > hx) hx = next_piece.map[i][0];
		if (next_piece.map[i][1] < ly) ly = next_piece.map[i][1];
		if (next_piece.map[i][1] > hy) hy = next_piece.map[i][1];
	}
	cx = (int)((float)(lx+(hx-lx+1)/2.0)*block_size - block_size*2);
	cy = (int)((float)(ly+(hy-ly+1)/2.0)*block_size - block_size*2);
	
	//outlines
	if ((*user_prefs)["block_outline"] != "off")
	{
		int olo = prefs->geti("outline_opacity");
		bool bw = (*user_prefs)["block_outline"] == "white";
		SDL_Rect ol_rect = {0, 0, block_size+2, block_size+2};
		for (int i = 0; i < 4; i++)
		{
			ol_rect.x = next_piece.map[i][0]*block_size-cx-1;
			ol_rect.y = next_piece.map[i][1]*block_size-cy-1;
			SDL_FillRect(next_piece_box, &ol_rect, SDL_MapRGBA(next_piece_box->format, 255*bw, 255*bw, 255*bw, olo));
		}
	}
	bool rmode = (*user_prefs)["rmode"] == "on";
	src.w = block_size;
	src.h = block_size;
	src.x = 0;
	src.y = 0;
	dest = src;
	block_type bt(next_piece.graphic, theme_id);
	SDL_SetAlpha(blocks[bt], 0, 0);
	for (int i = 0; i < 4; i++)
	{
		dest.x = next_piece.map[i][0]*block_size-cx;
		dest.y = next_piece.map[i][1]*block_size-cy;
		if (!rmode)
		{
			SDL_BlitSurface(blocks[bt], &src, next_piece_box, &dest);
		} else {
			SDL_FillRect(next_piece_box, &dest, SDL_MapRGBA(next_piece_box->format, 0, 0, 0, 0));
		}
	}
	SDL_SetAlpha(blocks[bt], SDL_SRCALPHA, 0);
	ls.findByName(string("next_piece"))->changed = true;
}

void Sporktris::clear_surf (SDL_Surface *surf)
{
	SDL_FillRect(surf, NULL, SDL_MapRGBA(surf->format, 0, 0, 0, 0));
}

piece Sporktris::rand_piece (void)
{
	//return(piece(piece_specs[0], prefs->geti("board_width")/2-2, -4, &board));
	piece my_piece(piece_specs[random(0, piece_specs.size()-1)], prefs->geti("board_width")/2-2, -4, &board);
	return(my_piece);
}

bool Sporktris::handle_events (SDL_Event *event)
{
	if (event->type == SDL_KEYDOWN || event->type == SDL_KEYUP)
	{
		keys.handle_event(event);
		return (0);
	}
	
	switch (event->type)
	{
		case SDL_ACTIVEEVENT:
		if (event->active.state == SDL_APPINPUTFOCUS || event->active.state == SDL_APPACTIVE)
		{
			if (event->active.gain == 0 && paused == false)
			{
				pause();
				return(0);
			}
		}
		break;
		
		case SDL_QUIT:
		return (1);
		break;
		
		default:
		return (0);
		break;
	}
	return (0);
}

int Sporktris::handle_input (void)
{
	if (keys["pause"].get_event())
	{
		pause();
		if (!paused)
		{
			return (1);
		} else {
			return (0);
		}
	}
	
	if (paused || awaiting_gravity)
	{
		return (0);
	}
	
	int result = 0;
	
	while (keys["turn_left"].get_event())
	{
		if (turn_piece(1)) result = 1;
	}
	
	while (keys["turn_right"].get_event())
	{
		if (turn_piece(0)) result = 1;
	}
	
	while (cur_piece.canMove(-1,0) && keys["move_left"].get_event())
	{
		move_piece(-1, 0);
		result = 1;
	}
	
	while (cur_piece.canMove(1,0) && keys["move_right"].get_event())
	{
		move_piece(1, 0);
		result = 1;
	}
	
	while (keys["move_down"].get_event())
	{
		if (cur_piece.canMove(0,1))
		{
			move_piece(0, 1);
			last_step = SDL_GetTicks();
			result = 1;
		}
	}
	
	if (keys["drop"].get_event())
	{
		if (!cur_piece.canMove(0,1) && cur_piece.y < 0)
		{
			result = 2;
		} else {
			drop_piece();
			last_step = SDL_GetTicks();
			result = 1;
		}
	}
	
	if (keys["end_game"].get_event())
	{
		result = 2;
	}
	
#ifdef DEBUG
	if (keys["skip_level"].get_event())
	{
		set_level(level+1);
		update_points();
		update_lines();
		update_level();
		update_next_piece();
	}
#endif
	
	return result;
}

void Sporktris::pause (void)
{
	layer *l;
	if (!paused)
	{
		#ifdef DEBUG
		if (!keys["debug_mod"].is_down())
		{
		#endif
			for (int x = 0; x < prefs->geti("board_width"); x++)
			{
				for (int y = 0; y < prefs->geti("board_height"); y++)
				{
					if (bufboard[y][x].type != -1)
					{
						bufboard[y][x].type = -1;
						l = ls.findByName(funcs::str_int(x)+":"+funcs::str_int(y));
						l->surf = NULL;
						l->changed = true;
					}
					l = ls.findByName(funcs::str_int(x)+":"+funcs::str_int(y)+"o");
					if (l->surf != NULL) l->changed = true;
					l->surf = NULL;
				}
			}
			l = ls.findByName("paused");
			l->surf = paused_img;
			l->changed = true;
			/*l = ls.findByName("block_outlines");
			l->surf = NULL;
			l->changed = true;*/
		#ifdef DEBUG
		}
		#endif
		ls.blit(screen);
		SDL_Flip(screen);
	} else {
		l = ls.findByName("paused");
		l->surf = NULL;
		l->changed = true;
		/*l = ls.findByName("block_outlines");
		l->surf = block_outlines;
		l->changed = true;*/
		last_step = SDL_GetTicks()-(last_step % speed);
		keys.reset_repeat();
	}
	paused ? paused = false : paused = true;
}

bool Sporktris::turn_piece (bool direction)
{
	piece old_piece = cur_piece;
	bool result = cur_piece.turn(direction);
	if (result)
	{
		for (int i = 0; i < 4; i++)
		{
			set_board(old_piece.x+old_piece.map[i][0], old_piece.y+old_piece.map[i][1], block_type(-1, theme_id));
		}
		for (int i = 0; i < 4; i++)
		{
			set_board(cur_piece.x+cur_piece.map[i][0], cur_piece.y+cur_piece.map[i][1], block_type(cur_piece.graphic, theme_id));
		}
	}
	return (result);
}

void Sporktris::move_piece (int x_change, int y_change)
{
	piece old_piece = cur_piece;
	cur_piece.x += x_change;
	cur_piece.y += y_change;
	for (int i = 0; i < 4; i++)
	{
		set_board(old_piece.x+old_piece.map[i][0], old_piece.y+old_piece.map[i][1], block_type(-1, theme_id));
	}
	for (int i = 0; i < 4; i++)
	{
		set_board(cur_piece.x+cur_piece.map[i][0], cur_piece.y+cur_piece.map[i][1], block_type(cur_piece.graphic, theme_id));
	}
}

void Sporktris::update_win (void)
{
	layer *l;
	bool rmode = (*user_prefs)["rmode"] == "on";
	vector< pair<int, int> > to_update;
	int bw = prefs->geti("board_width");
	int bh = prefs->geti("board_height");
	for (int y = 0; y < prefs->geti("board_height"); y++)
	{
		for (int x = 0; x < prefs->geti("board_width"); x++)
		{
			if (board[y][x] != bufboard[y][x])
			{
				bufboard[y][x] = board[y][x];
				l = ls.findByName(funcs::str_int(x)+string(":")+funcs::str_int(y));
				if (board[y][x].type != -1 && !rmode)
				{
					block_type bt = board[y][x];
					bt.type = bt.type % themes.size();
					l->surf = blocks[bt];
				} else {
					l->surf = NULL;
				}
				l->changed = true;
				if ((*user_prefs)["block_outline"] != "off")
					for (int y2 = max(y-1, 0); y2 <= min(y+1, bh-1); y2++)
						for (int x2 = max(x-1, 0); x2 <= min(x+1, bw-1); x2++)
						{
							pair<int, int> p(y2, x2);
							bool do_it = true;
							for (unsigned int i = 0; i < to_update.size(); i++)
								if (p == to_update[i]) {do_it = false; break;}
							if (do_it) to_update.push_back(p);
						}
			}
			/*
			if ((*user_prefs)["block_outline"] != "off" && board[y][x].type != -1)
			{
				int bs = prefs->geti("block_size");
				SDL_Rect r = {x*bs, y*bs, bs+2, bs+2};
				Uint8 c = (*user_prefs)["block_outline"] == "black" ? 0 : 255;
				SDL_FillRect(block_outlines, &r, SDL_MapRGBA(block_outlines->format, c, c, c, 96));
			}*/
		}
	}
	if ((*user_prefs)["block_outline"] != "off")
		update_outlines(to_update);
	ls.blit(screen);
	SDL_Flip(screen);
}

void Sporktris::update_level (void)
{
	SDL_Rect box, dest;
	SDL_GetClipRect(level_box, &box);
	SDL_Rect letter = {prefs->geti("level_font_x"), prefs->geti("level_font_y"), prefs->geti("level_font_w"), prefs->geti("level_font_h")};
	dest = letter;
	dest.y = (box.h/2)-(letter.h/2);
	dest.x = (box.w/2)-letter.w;
	clear_surf(level_box);
	if (level >= 10)
	{
		letter.y = (int)(level/10)*letter.h;
		SDL_BlitSurface(level_theme, &letter, level_box, &dest);
		dest.x += letter.w;
	} else {
		dest.x += letter.w/2;
	}
	letter.y = (int)(level%10)*(letter.h);
	SDL_BlitSurface(level_theme, &letter, level_box, &dest);
	ls.findByName("level")->changed = true;
}

void Sporktris::update_points (void)
{
	blit_nums(points, points_box);
	ls.findByName("points")->changed = true;
}

void Sporktris::update_lines (void)
{
	blit_nums(cleared_lines, lines_box);
	ls.findByName("lines")->changed = true;
}

void Sporktris::blit_nums (int num, SDL_Surface *destSurf)
{
	string numStr = funcs::str_int(num);
	SDL_Rect box, dest;
	SDL_GetClipRect(destSurf, &box);
	SDL_Rect letter = {prefs->geti("small_num_font_x"), prefs->geti("small_num_font_y"), prefs->geti("small_num_font_w"), prefs->geti("small_num_font_h")};
	clear_surf(destSurf);
	dest = letter;
	dest.x = destSurf->w;
	dest.y = (destSurf->h/2)-(dest.h/2);
	for (int i = numStr.size()-1; i > -1; i--)
	{
		dest.x -= dest.w;
		letter.y = (atoi(numStr.substr(i, 1).c_str()))*dest.h;
		SDL_BlitSurface(level_theme, &letter, destSurf, &dest);
	}
}

Sporktris::Sporktris (Namedv *prefs_ptr, Namedv *user_prefs_ptr)
{
	srand(static_cast<unsigned>(time(0)));
	screen = SDL_GetVideoSurface();
	paused = false;
	awaiting_gravity = false;
	level = 1;
	theme_id = 0;
	points = 0;
	cleared_lines = 0;
	speed = base_speed;
	prefs = prefs_ptr;
	user_prefs = user_prefs_ptr;
	load_pieces();
	load_surfs();
	init_board();
	init_layers();
}

Sporktris::~Sporktris (void)
{
	SDL_FreeSurface(game_over);
	SDL_FreeSurface(level_box);
	SDL_FreeSurface(points_box);
	SDL_FreeSurface(lines_box);
	SDL_FreeSurface(next_piece_box);
	SDL_FreeSurface(paused_img);
	for (map<block_type, SDL_Surface*>::iterator iter = blocks.begin(); iter != blocks.end(); iter++)
		SDL_FreeSurface(iter->second);

	for (unsigned int i = 0; i < themes.size(); i++)
		SDL_FreeSurface(themes[i]);
	
	for (map<unsigned char, SDL_Surface*>::iterator iter = outline_surfs_b.begin(); iter != outline_surfs_b.end(); iter++)
		SDL_FreeSurface(iter->second);
	
	for (map<unsigned char, SDL_Surface*>::iterator iter = outline_surfs_w.begin(); iter != outline_surfs_w.end(); iter++)
		SDL_FreeSurface(iter->second);
}

void Sporktris::load_pieces (void)
{
	ifstream file((*prefs)["pieces_file"].c_str());
	string line;
	while (getline(file, line) != 0)
	{
		piece_specs.push_back(line);
	}
}

void Sporktris::init_keys (void)
{
	keys.clear();
	string key_names[] = {"pause", "turn_left", "turn_right", "move_left", "move_right", "move_down", "drop", "end_game"};
	for (unsigned int i = 0; i < sizeof(key_names)/sizeof(string); i++)
	{
		int delay = prefs->geti("kr_delay");
		if ((*prefs)[string("krd_")+key_names[i]] != "")
			delay = prefs->geti(string("krd_")+key_names[i]);
		keys.add_key(key_names[i], Key((*user_prefs)[key_names[i]+"_key"], delay, prefs->geti(string("kr_")+key_names[i])));
	}
	
	//is only used if DEBUG is defined
	keys.add_key("skip_level", Key("n", 150, 130));
	keys.add_key("debug_mod", Key("a", 0, 0));
}

void Sporktris::init_board (void)
{
	vector<block_type> temp;
	for (int i = 0; i < prefs->geti("board_width"); i++)
	{
		temp.push_back(block_type(-1, theme_id));
	}
	for (int y = 0; y < prefs->geti("board_height"); y++)
	{
		bufboard.push_back(temp);
		board.push_back(temp);
	}
}

void Sporktris::load_surfs (void)
{
	for (int i = 1; (*prefs)[string("level_theme")+funcs::str_int(i)] != ""; i++)
	{
		string name = string("level_theme")+funcs::str_int(i);
		themes.push_back(funcs::load_surf((*prefs)[name], 1, name.c_str()));
	}
	level_theme = themes[0];
	game_over = funcs::load_surf((*prefs)["game_over_img"], 2, "game over image");
	paused_img = funcs::load_surf((*prefs)["paused_img"], 2, "paused image");
	
	SDL_Surface *bw_block = funcs::load_surf((*prefs)["block_img"], 1, "block image");
	color_blocks(bw_block);
	SDL_FreeSurface(bw_block);
	
	next_piece_box = funcs::load_surf(prefs->geti("block_size")*4, prefs->geti("block_size")*4, "next piece box");
	points_box = funcs::load_surf(prefs->geti("points_w"), prefs->geti("points_h"), "points box");
	lines_box = funcs::load_surf(prefs->geti("lines_w"), prefs->geti("lines_h"), "lines box");
	level_box = funcs::load_surf(prefs->geti("level_w"), prefs->geti("level_h"), "level box");
	int bs = prefs->geti("block_size");
	int olo = prefs->geti("outline_opacity");
	for (int i = 0; i < 256; i++)
	{
		for (int c = 0; c <= 1; c++)
		{
			SDL_Surface *surf = funcs::load_surf(prefs->geti("block_size"), prefs->geti("block_size"), "a block outline surf");
			clear_surf(surf);
			unsigned char key = 1;
			for (int y = -1; y <= 1; y++)
			{
				for (int x = -1; x <= 1; x++)
				{
					if (y == 0 && x == 0) continue;
					if (i & key)
					{
						SDL_Rect r = {(x == 1 ? bs-1 : 0), (y == 1 ? bs-1 : 0), (x ? 1 : bs), (y ? 1 : bs)};
						SDL_FillRect(surf, &r, SDL_MapRGBA(surf->format, c*255, c*255, c*255, olo));
					}
					key *= 2;
				}
			}
			(c ? outline_surfs_w : outline_surfs_b)[(char)i] = surf;
		}
	}
}

void Sporktris::init_layers (void)
{
	ls.add(layer("background", level_theme, 0, 0));
	ls.add(layer("next_piece", next_piece_box, prefs->geti("next_piece_x"), prefs->geti("next_piece_y")));
	ls.add(layer("level", level_box, prefs->geti("level_x"), prefs->geti("level_y")));
	ls.add(layer("points", points_box, prefs->geti("points_x"), prefs->geti("points_y")));
	ls.add(layer("lines", lines_box, prefs->geti("lines_x"), prefs->geti("lines_y")));
	SDL_Rect rect = {0, 0, prefs->geti("block_size"), prefs->geti("block_size")};
	for (int y = 0; y < prefs->geti("board_height"); y++)
	{
		for (int x = 0; x < prefs->geti("board_width"); x++)
		{
			ls.add(layer(funcs::str_int(x)+":"+funcs::str_int(y)+"o", NULL, prefs->geti("board_x")+x*prefs->geti("block_size"), prefs->geti("board_y")+y*prefs->geti("block_size")));
			ls.add(layer(funcs::str_int(x)+":"+funcs::str_int(y), NULL, prefs->geti("board_x")+x*prefs->geti("block_size"), prefs->geti("board_y")+y*prefs->geti("block_size"), rect));
		}
	}
	ls.add(layer(string("paused"), NULL, (prefs->geti("board_x")+(prefs->geti("board_width")*prefs->geti("block_size"))/2)-paused_img->w/2, (prefs->geti("board_y")+(prefs->geti("board_height")*prefs->geti("block_size"))/2)-paused_img->h/2));
}

void Sporktris::reset (void)
{
	paused = false;
	awaiting_gravity = false;
	level = 1;
	theme_id = 0;
	points = 0;
	cleared_lines = 0;
	speed = base_speed;
	if ((unsigned)level <= themes.size())
	{
		level_theme = themes[level-1];
		ls.findByName("background")->surf = level_theme;
	}
	for (int y = 0; y < prefs->geti("board_height"); y++)
	{
		for (int x = 0; x < prefs->geti("board_width"); x++)
		{
			board[y][x].type = -1;
			bufboard[y][x].type = -1;
			ls.findByName(funcs::str_int(x)+":"+funcs::str_int(y))->surf = NULL;
			ls.findByName(funcs::str_int(x)+":"+funcs::str_int(y)+"o")->surf = NULL;
		}
	}
	for (unsigned int i = 0; i < ls.layers.size(); i++)
	{
		ls.layers[i].changed = true;
	}
}

int Sporktris::random(int min, int max)
{
	if (min > max)
	{
    	swap(min, max);
    }
	int range = max - min + 1;
	int rando = rand();
	int ret = min + int(range * (rando/(RAND_MAX + 1.0)));
	return ret;
}

void Sporktris::color_blocks (SDL_Surface *block)
{
	int bpp = block->format->BytesPerPixel;
	
	//get min and max value for the block
	Uint8 bmin = 255;
	Uint8 bmax = 0;
	for (int y = 0; y < block->h; y++)
	{
		for (int x = 0; x < block->w; x++)
		{
			Uint32 *p = (Uint32*)((Uint8*)block->pixels + y*block->pitch + x*bpp);
			SDL_Color color;
			SDL_GetRGBA(*p, block->format, &color.r, &color.g, &color.b, &color.unused);
			Uint8 bmax2 = color.r;
			if (color.g > bmax2) bmax2 = color.g;
			if (color.b > bmax2) bmax2 = color.b;
			if (bmax2 < bmin) bmin = bmax2;
			if (bmax2 > bmax) bmax = bmax2;
		}
	}
	
	for (unsigned int l = 0; l < themes.size(); l++)
	{
		int tbpp = themes[l]->format->BytesPerPixel;
		Uint32 *theme_pixel = (Uint32*)((Uint8*)themes[l]->pixels + prefs->geti("block_color_y")*themes[l]->pitch + prefs->geti("block_color_x")*tbpp);
		SDL_Color tc_rgb;
		SDL_GetRGBA(*theme_pixel, themes[l]->format, &tc_rgb.r, &tc_rgb.g, &tc_rgb.b, &tc_rgb.unused);
		int tc = max(max(tc_rgb.r, tc_rgb.g), tc_rgb.b);
		//cerr << endl << tc.h << " : " << tc.s << " : " << tc.v << endl;
		for (int i = 0; i < 7; i++)
		{
			SDL_Surface *c_block = SDL_DisplayFormatAlpha(block);
			block_type bt(i, l);
			for (int y = 0; y < c_block->h; y++)
			{
				for (int x = 0; x < c_block->w; x++)
				{
					Uint32 *p = (Uint32*)((Uint8*)c_block->pixels + y*c_block->pitch + x*bpp);
					SDL_Color color;
					SDL_GetRGBA(*p, c_block->format, &color.r, &color.g, &color.b, &color.unused);
					int bc = max(max(color.r, color.g), color.b);
					float b = bc + tc - bmin - (bmax-bmin)/2 + (-3.5+i)*prefs->geti("block_variation");
					//if (b > 255) b = 255;
					if (b < 0) b = 0;
					color.r = min(int(b * tc_rgb.r/255), 255);
					color.g = min(int(b * tc_rgb.g/255), 255);
					color.b = min(int(b * tc_rgb.b/255), 255);
					*p = SDL_MapRGBA(c_block->format, color.r, color.g, color.b, color.unused);
				}
			}
			blocks[bt] = c_block;
		}
	}
}

void Sporktris::make_garbage (int height)
{
	block_type bt(1, level-1);
	int bh = prefs->geti("board_height");
	int bw = prefs->geti("board_width");
	for (int y = 0; y <= height; y++)
	{
		int amount = random(bw/2, bw-1);
		for (int i = 0; i < amount; i++)
		{
			int x = random(0, bw-1);
			if (board[y][x].type != -1) { i--; continue; }
			bt.type = random(0, piece_specs.size()-1);
			set_board(x, bh-y, bt);
		}
	}
}

void Sporktris::update_outlines (const vector< pair<int, int> >& to_update)
{
	int bw = prefs->geti("board_width");
	int bh = prefs->geti("board_height");
	bool c = (*user_prefs)["block_outline"] == "white";
	for (unsigned int i = 0; i < to_update.size(); i++)
	{
		layer *l = ls.findByName(funcs::str_int(to_update[i].second)+":"+funcs::str_int(to_update[i].first)+"o");
		unsigned char key = 0;
		if (board.at(to_update[i].first).at(to_update[i].second).type == -1)
		{
			unsigned char key2 = 1;
			for (int y = to_update[i].first-1; y <= to_update[i].first+1; y++)
			{
				for (int x = to_update[i].second-1; x <= to_update[i].second+1; x++)
				{
					int rely = y-to_update[i].first;
					int relx = x-to_update[i].second;
					if (rely == 0 && relx == 0) continue;
					if (y >= 0 && x >= 0 && y < bh && x < bw && board.at(y).at(x).type != -1)
					{
						key |= key2;
					}
					key2 *= 2;
				}
			}
		}
		if (l->surf != (c ? outline_surfs_w : outline_surfs_b)[key])
		{
			l->surf = (c ? outline_surfs_w : outline_surfs_b)[key];
			l->changed = true;
		}
	}
}
